@@ -778,6 +778,10 @@ int main(int ac, char** av)
               if (--ac == 0)
                 exit_with_usage(myname, "Specify session-name with -S", NULL);
               SockMatch = *++av;
+              debug1("SockMatch: '%s'\n", SockMatch);
+              debug1("SockMatch len: '%d'\n", (int)strlen(SockMatch));
+              if (strlen(SockMatch) > 80)
+                exit_with_usage(myname, "Session-name is too long (max length is 80 symbols)", NULL);
             }
             if (!*SockMatch)
               exit_with_usage(myname, "Empty session-name?", NULL);
@@ -1191,11 +1195,25 @@ int main(int ac, char** av)
     eexit(0);
   }
   signal(SIG_BYE, AttacherFinit);	/* prevent races */
+
   if (cmdflag) {
+
+#ifdef MULTIUSER
+    if (multi)
+      real_uid = multi_uid;
+#endif
+
     /* attach_tty is not mandatory */
     SetTtyname(false, &st);
     if (!*av)
       Panic(0, "Please specify a command.");
+    if (!strncmp("sessionname", *av, 11)) {
+      if (!*++av)
+        Panic(0, "Please specify a parameter.");
+      if (strlen(*av) > 80)
+        Panic(0, "Parameter of command 'sessionname' is too long.");
+    *--av;
+    }
     SET_GUID();
     SendCmdMessage(sty, SockMatch, av, queryflag >= 0);
     exit(0);
@@ -2755,6 +2773,18 @@ char *MakeWinMsgEv(char *str, struct win *win, int esc, int padlen, struct event
       p += strlen(p) - 1;
       break;
 
+#ifdef ENCODINGS
+    case 'e':
+      *p = 0;
+      D_encoding = nwin_options.encoding > 0 ? nwin_options.encoding : 0;
+      if (win && win->w_encoding) {
+        *p++ = ' ';
+        strcpy(p, EncodingName(win->w_encoding));
+      }
+      p += strlen(p) - 1;
+      break;
+#endif
+
     case '{':
     {
       char rbuf[128];
